@@ -50,7 +50,7 @@ class provider():
 	def sendSMS(self, app, NUMBER, SMSTEXT, LOGIN, PASSWORD):
 		return False, "SMS sending not implemented"
 
-# o2online.de 2.2
+# o2online.de 2.3
 # o2online (DE) Provider written by Bjoern Olausson <maemo@olausson.de>
 class provider_o2online_de(provider):
 	def getName(self):
@@ -198,14 +198,14 @@ class provider_o2online_de(provider):
 			return SCHEDULE, SENDER, FLASH
 
 	def sendSMS(self, MainWindow, app, NUMBER, SMSTEXT, LOGIN, PASSWORD):
-		LOGINURL = 'https://o2online.de/'
+		LOGINURL = 'https://o2online.de'
 		execute = True
 		if execute:
 			# Get the values for the advanced settings.
 			SCHEDULE, SENDER, FLASH = self.get_advanced_setting(MainWindow)
 			MainWindow.Stat.show()
 			MainWindow.Stat.ui.progressBar.setMinimum(0)
-			MainWindow.Stat.ui.progressBar.setMaximum(11)
+			MainWindow.Stat.ui.progressBar.setMaximum(9)
 			# RegExp to check if SMS was send successfully:
 			re_suc = re.compile('erfolgreich', re.IGNORECASE)
 			# RegEx to check if en error occured
@@ -216,11 +216,11 @@ class provider_o2online_de(provider):
 			re_free = re.compile('Frei-SMS: [0-9]+')
 
 			# Instanziate a Browser
-			UpdatePbar(self, MainWindow, app, "1", "Initiating the browser")
+			UpdatePbar(self, MainWindow, app, "1", "Step 1: Initiating the browser")
 			try:
 				br = mechanize.Browser()
 			except Exception, e:
-				return error(self, MainWindow, e, "Failed to initiate the Browser")
+				return error(self, MainWindow, e, "Step 1: Failed to initiate the Browser")
 
 			# Browser options
 			# gzip transfer encoding is experimental! --> Disable it
@@ -250,100 +250,85 @@ class provider_o2online_de(provider):
 			# Opera
 			br.addheaders = [('User-agent', 'Opera/9.80 (X11; Linux x86_64; U; en) Presto/2.5.24 Version/10.54 Gentoo')]
 
+			#------------------------------------
 			# Open the O2 website
-			UpdatePbar(self, MainWindow, app, "2", "Open the O2 website")
-			br.open(LOGINURL)
-			response = br.response().read()
+			#------------------------------------
+			UpdatePbar(self, MainWindow, app, "2", "Step 2: Open the O2 website")
+			try:
+				br.open(LOGINURL)
+				response = br.response().read()
+			except Exception, e:
+				return error(self, MainWindow, e, 'Step 2: Failed to open %s' %(LOGINURL))
 
-			# Select the Login form
-			UpdatePbar(self, MainWindow, app, "3", "Log in to MyO2")
+			#------------------------------------
+			# Select the Login form and feed it with the login credentials
+			#------------------------------------
+			UpdatePbar(self, MainWindow, app, "3", "Step 3: Log in to MyO2")
 			try:
 				br.select_form(name='loginForm')
 			except Exception, e:
-				return error(self, MainWindow, e, "Faild to select the login Form")
+				return error(self, MainWindow, e, "Step 3: Faild to select the login Form")
 			br.form['loginName'] = LOGIN
 			br.form['password'] = PASSWORD
-
-			# The dropdown list entry for the URL below has the name "Mein o2"
-			# So how can I set the selected item via the name instead of the value like:
-			# br.form["url"] = ["Mein o2"]
-			#for item in br.form.find_control("url", type="select").items:
-			#    print item
-			br.form["url"] = ["http://www.o2online.de/nw/meino2/meino2.html?et_cid=22&et_lid=61"]
-
+			#br.find_control(name="url").value = ["1"]
+			#br.form.find_control(name="url", kind="select").value = ["1]
+			#br.form['url'] = ["https://email.o2online.de/ssomanager.osp?APIID=AUTH-WEBSSO&#38;et_cid=22&#38;et_lid=64"]
 			try:
 				br.submit()
 				response = br.response().read()
 			except Exception, e:
-				return error(self, MainWindow, e, 'Failed to load "Mein O2"')
+				return error(self, MainWindow, e, 'Step 3: Failed to load "Mein O2"')
 			if re_fail.search(str(response)):
-				return error(self, MainWindow, False, 'Probably wrong login parameters, check your account setup')
+				return error(self, MainWindow, False, 'Step 3: Probably wrong login parameters, check your account setup')
 
+			#------------------------------------
 			#Check for CAPTCHA request
-			UpdatePbar(self, MainWindow, app, "4", "CAPTCHA-Check")
+			#------------------------------------
+			UpdatePbar(self, MainWindow, app, "4", "Step 4: CAPTCHA-Check")
 			self.check_captcha(response)
 
-			#Find the Link to the SMS-Center
-			#smscenter = br.find_link(text_regex="SMS versenden \(online\)")
-			UpdatePbar(self, MainWindow, app, "5", "Open the SMS-Center")
+			#------------------------------------
+			# Open the EMAIL-Center
+			#------------------------------------
+			UpdatePbar(self, MainWindow, app, "5", "Step 5: Open EmailCenter")
 			try:
-				br.follow_link(text_regex="SMS versenden \(online\)")
+				br.find_link(text='E-Mails bearbeiten')
+			except mechanize.LinkNotFoundError:
+				return error(self, MainWindow, e, 'Step 5: Link "E-Mails bearbeiten" nicht gefunden')
+			req = br.click_link(text='E-Mails bearbeiten')
+			try:
+				br.open(req)
 				response = br.response().read()
 			except Exception, e:
-				return error(self, MainWindow, e, 'Failed to open the link for the SMS-Center')
-
-			#Follow the JavaScript redirect manually
-			UpdatePbar(self, MainWindow, app, "6", "Following the redirect")
-			redirect = re.findall(r'loc = \'http://.+\'', response)
-			if not redirect:
-				return error(self, MainWindow, e, 'Failed to find the redirect to the SMS-Center')
-
-			redirect = redirect[0].split()[2].strip('\'')
-			try:
-				br.open(redirect)
-				response = br.response().read()
-			except Exception, e:
-				return error(self, MainWindow, e, 'Failed to follow the redirect to the SMS-Center')
-
-			# Select the Login form (second time...)
-			UpdatePbar(self, MainWindow, app, "7", "Log in to the SMS-Center")
-			try:
-				br.select_form(name='login')
-			except Exception, e:
-				return error(self, MainWindow, e, 'Failed to select the second login form')
-			br.form['loginName'] = LOGIN
-			br.form['password'] = PASSWORD
-			try:
-				br.submit()
-				response = br.response().read()
-			except Exception, e:
-				return error(self, MainWindow, e, 'Faild for the second login')
-			if re_fail.search(str(response)):
-				return error(self, MainWindow, False, 'Probably wrong login parameters, check your account setup')
-
-			#Check for CAPTCHA request
-			UpdatePbar(self, MainWindow, app, "8", "CAPTCHA-Check")
-			self.check_captcha(response)
+				return error(self, MainWindow, e, 'Step 5: Failed to open the EmailCenter')
 
+			#------------------------------------
 			# Open the SMS-Center
-			UpdatePbar(self, MainWindow, app, "9", "Open the SMS-Center")
+			#------------------------------------
+			UpdatePbar(self, MainWindow, app, "6", "Step 6: Open SMSCenter")
+			req = 'https://email.o2online.de/smscenter_new.osp?Autocompletion=1&MsgContentID=-1'
 			try:
-				br.open('https://email.o2online.de/smscenter_new.osp?Autocompletion=1&MsgContentID=-1')
+				br.open(req)
 				response = br.response().read()
 			except Exception, e:
-				return error(self, MainWindow, e, 'Failed to open the SMS-Center')
+				return error(self, MainWindow, e, 'Step 6: Failed to open the SMSCenter')
+
+			#------------------------------------
+			# Select the SMS-Form
+			#------------------------------------
+			UpdatePbar(self, MainWindow, app, "7", "Step 7: Select the SMS-Form")
+			try:
+				br.select_form(name='frmSMS')
+			except Exception, e:
+				return error(self, MainWindow, e, 'Step 7: Failed to select the SMS form')
 
 			# Find how much FREE SMS we have left.
 			FREESMS = int(re_free.findall(response)[0].split()[1])
 
-			# Select the SMS-Form
-			try:
-				br.select_form(name='frmSMS')
-			except Exception, e:
-				return error(self, MainWindow, e, 'Failed to select the SMS form')
-
+			#------------------------------------
 			# Fill the forms
-			UpdatePbar(self, MainWindow, app, "10", "Fill the forms")
+			#------------------------------------
+			UpdatePbar(self, MainWindow, app, "8", "Step 8: Fill the forms")
 			br.form.set_all_readonly(False)
 			# FIll in number
 			br.form['SMSTo'] = NUMBER
@@ -375,8 +360,11 @@ class provider_o2online_de(provider):
 				if SCHEDULE[0][1] :
 					br.form['RepeatEndType'] = '1'
 					br.form['RepeatEndDate'] = StopDate
+
+			#------------------------------------
 			# Finally send the SMS
-			UpdatePbar(self, MainWindow, app, "11", "Send SMS")
+			#------------------------------------
+			UpdatePbar(self, MainWindow, app, "9", "Step 9: Send SMS")
 			try:
 				#print NUMBER, SMSTEXT, LOGIN, PASSWORD, SCHEDULE, SENDER, FLASH
 				br.submit()
@@ -385,9 +373,9 @@ class provider_o2online_de(provider):
 					SMS_LEFT = FREESMS - 1
 					return error(self, MainWindow, True, 'Provider responded: Ihre SMS wurde erfolgreich versendet\nVerbleibende SMS: %s' %(SMS_LEFT))
 				else:
-					return error(self, MainWindow, False, 'Provider responded: Ihre SMS wurde nicht versendet\nVerbleibende SMS: %s' %(FREESMS))
+					return error(self, MainWindow, False, 'Step 9: Provider responded: Ihre SMS wurde nicht versendet\nVerbleibende SMS: %s' %(FREESMS))
 			except Exception, e:
-					return error(self, MainWindow, e, 'Submission faild')
+					return error(self, MainWindow, e, 'Step 9: Submission faild')
 
 # vodafone.nl 1.0
 # Vodafone (NL) Provider written by Shane Kerr <shane@time-travellers.org>
